<?php

namespace Ebuy\Cart\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Ebuy\Cart\Forms\Cart as CartForm;
use Ebuy\Cart\Http\Requests\CartRequest;
use Ebuy\Cart\Interfaces\CartRepositoryInterface;
use Ebuy\Cart\Repositories\Eloquent\Filters\CartResourceFilter;
use Ebuy\Cart\Repositories\Eloquent\Presenters\CartListPresenter;

/**
 * Resource controller class for cart.
 */
class CartResourceController extends BaseController
{

    /**
     * Initialize cart resource controller.
     *
     *
     * @return null
     */
    public function __construct(CartRepositoryInterface $cart)
    {
        parent::__construct();
        $this->form = CartForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('ebuy.cart.modules'), 'cart', guard_url('cart'));
        $this->repository = $cart;
    }

    /**
     * Display a list of cart.
     *
     * @return Response
     */
    public function index(CartRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(CartResourceFilter::class)
            ->setPresenter(CartListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('cart::cart.names'))
            ->view('cart::cart.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display cart.
     *
     * @param Request $request
     * @param Model   $cart
     *
     * @return Response
     */
    public function show(CartRequest $request, CartRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('cart::cart.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('cart::cart.show')
            ->output();
    }

    /**
     * Show the form for creating a new cart.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(CartRequest $request, CartRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('cart::cart.name'))
            ->view('cart::cart.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new cart.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CartRequest $request, CartRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('cart::cart.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('cart/cart/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/cart/cart'))
                ->redirect();
        }

    }

    /**
     * Show cart for editing.
     *
     * @param Request $request
     * @param Model   $cart
     *
     * @return Response
     */
    public function edit(CartRequest $request, CartRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('cart::cart.name'))
            ->view('cart::cart.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the cart.
     *
     * @param Request $request
     * @param Model   $cart
     *
     * @return Response
     */
    public function update(CartRequest $request, CartRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('cart::cart.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('cart/cart/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('cart/cart/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the cart.
     *
     * @param Model   $cart
     *
     * @return Response
     */
    public function destroy(CartRequest $request, CartRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('cart::cart.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('cart/cart/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('cart/cart/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
